# ------------------------------------------------------------------
# example 1: NS
# ------------------------------------------------------------------

# set up yield curve (in this example: artificial data), and plot it
mats <- c(1,3,6,9,12,15,18,21,24,30,36,48,60,72,84,96,108,120)/12
betaTRUE <- c(6,3,8,1); yM <- NS(betaTRUE,mats)
plot(mats,yM,xlab="maturities in years",ylab="yields in %")

# collect all data in dataList
dataList <- list(yM = yM, mats = mats, model = NS)

# set parameters for de
de <- list(
		min	= c( 0,-15,-30,0),
		max	= c(15, 30, 30,10),
		d	= 4,
		nP	= 200,
		nG	= 600,
		ww	= 0.1,
		F	= 0.50,
		CR	= 0.99,
		R	= 0.00 # random term (added to change)
)

system.time(sol <- DE(de = de,dataList = dataList,OF = OF))
# maximum error
max(abs(dataList$model(sol$beta,mats)-dataList$model(betaTRUE,mats)))
# value of objective function
sqrt(sol$OFvalue)
lines(mats,dataList$model(sol$beta,mats), col="blue")

s0 <- de$min + (de$max - de$min) * runif(de$d)
system.time(sol2 <- nlminb(s0,OF,data=dataList,lower = de$min, upper = de$max, control = list(eval.max=50000,iter.max=50000)))
# maximum error
max(abs(dataList$model(sol2$par,mats)-dataList$model(betaTRUE,mats)))
# value of objective function
sqrt(sol2$objective)	
lines(mats,dataList$model(sol2$par,mats), col="green", lty=2)

legend(x  = "bottom", legend = c("true yields","DE","nlminb"),
	col = c("black","blue","green"),
	pch = c(1,NA,NA), lty=c(0,1,2))




# ------------------------------------------------------------------
# example 2: NSS
# ------------------------------------------------------------------

# set up yield curve (in this example: artificial data), and plot it
mats <- c(1,3,6,9,12,15,18,21,24,30,36,48,60,72,84,96,108,120)/12
betaTRUE <- c(5,-2,5,-5,1,3); yM <- NSS2(betaTRUE,mats) 
plot(mats,yM,xlab="maturities in years",ylab="yields in %")

# collect all in dataList
dataList <- list(yM = yM, mats = mats, model = NSS2)

# set parameters for de
de <- list(
		min	= c( 0,-15,-30,-30,0  ,2.5),
		max	= c(15, 30, 30, 30,2.5,5  ),
		d	= 6,
		nP	= 200,
		nG	= 600,
		ww	= 0.1,
		F	= 0.50,
		CR	= 0.99,
		R	= 0.00 # random term (added to change)
)

system.time(sol <- DE(de = de,dataList = dataList,OF = OF))
# maximum error
max(abs(dataList$model(sol$beta,mats)-dataList$model(betaTRUE,mats)))
# value of objective function
sqrt(sol$OFvalue)
lines(mats,dataList$model(sol$beta,mats), col="blue")

s0 <- de$min + (de$max - de$min) * runif(de$d)
system.time(sol2 <- nlminb(s0,OF,data=dataList,lower = de$min, upper = de$max, control = list(eval.max=50000,iter.max=50000)))
# maximum error
max(abs(dataList$model(sol2$par,mats)-dataList$model(betaTRUE,mats)))
# value of objective function
sqrt(sol2$objective)	
lines(mats,dataList$model(sol2$par,mats), col="green", lty=2)

legend(x  = "bottom", legend = c("true yields","DE","nlminb"),
		col = c("black","blue","green"),
		pch = c(1,NA,NA), lty=c(0,1,2))

